@@ -201,8 +201,10 @@ def init_anat_derivatives_wf(
     bids_root,
     freesurfer,
     num_t1w,
+    t2w,
     output_dir,
     spaces,
+    cifti_output,
     name="anat_derivatives_wf",
     tpm_labels=BIDS_TISSUE_ORDER,
 ):
@@ -223,6 +225,8 @@ def init_anat_derivatives_wf(
         Workflow name (default: anat_derivatives_wf)
     tpm_labels : :obj:`tuple`
         Tissue probability maps in order
+    cifti_output : :obj:`bool`
+        Whether the ``--cifti-output`` flag was set.
 
     Inputs
     ------
@@ -241,6 +245,8 @@ def init_anat_derivatives_wf(
         Segmentation in T1w space
     t1w_tpms
         Tissue probability maps in T1w space
+    t2w_preproc
+        The preprocessed T2w image, bias-corrected and resampled into anatomical space.
     anat2std_xfm
         Nonlinear spatial transform to resample imaging data given in anatomical space
         into standard space.
@@ -264,10 +270,18 @@ def init_anat_derivatives_wf(
         GIFTI surfaces (gray/white boundary, midthickness, pial, inflated)
     morphometrics
         GIFTIs of cortical thickness, curvature, and sulcal depth
+    anat_ribbon
+        Cortical ribbon volume in T1w space
     t1w_fs_aseg
         FreeSurfer's aseg segmentation, in native T1w space
     t1w_fs_aparc
         FreeSurfer's aparc+aseg segmentation, in native T1w space
+    cifti_morph
+        Morphometric CIFTI-2 dscalar files
+    cifti_density
+        Grayordinate density
+    cifti_metadata
+        JSON files containing metadata dictionaries
 
     """
     from niworkflows.interfaces.utility import KeySelect
@@ -284,14 +298,19 @@ def init_anat_derivatives_wf(
                 "t1w_mask",
                 "t1w_dseg",
                 "t1w_tpms",
+                "t2w_preproc",
                 "anat2std_xfm",
                 "std2anat_xfm",
                 "t1w2fsnative_xfm",
                 "fsnative2t1w_xfm",
                 "surfaces",
                 "morphometrics",
+                "anat_ribbon",
                 "t1w_fs_aseg",
                 "t1w_fs_aparc",
+                'cifti_metadata',
+                'cifti_density',
+                'cifti_morph',
             ]
         ),
         name="inputnode",
@@ -558,6 +577,27 @@ def init_anat_derivatives_wf(
     if not freesurfer:
         return workflow
 
+    # T2w coregistration requires FreeSurfer surfaces, so only try to save if freesurfer
+    if t2w:
+        ds_t2w_preproc = pe.Node(
+            DerivativesDataSink(
+                base_directory=output_dir,
+                desc="preproc",
+                suffix="T2w",
+                compress=True,
+            ),
+            name="ds_t2w_preproc",
+            run_without_submitting=True,
+        )
+        ds_t2w_preproc.inputs.SkullStripped = False
+        ds_t2w_preproc.inputs.source_file = t2w
+
+        # fmt:off
+        workflow.connect([
+            (inputnode, ds_t2w_preproc, [('t2w_preproc', 'in_file')]),
+        ])
+        # fmt:on
+
     from niworkflows.interfaces.nitransforms import ConcatenateXFMs
     from niworkflows.interfaces.surf import Path2BIDS
 
@@ -612,6 +652,19 @@ def init_anat_derivatives_wf(
         name="ds_morphs",
         run_without_submitting=True,
     )
+    # Ribbon volume
+    ds_anat_ribbon = pe.Node(
+        DerivativesDataSink(
+            base_directory=output_dir,
+            desc="ribbon",
+            suffix="mask",
+            extension=".nii.gz",
+            compress=True,
+        ),
+        name="ds_anat_ribbon",
+        run_without_submitting=True,
+    )
+
     # Parcellations
     ds_t1w_fsaseg = pe.Node(
         DerivativesDataSink(
@@ -650,8 +703,32 @@ def init_anat_derivatives_wf(
                                     ('source_files', 'source_file')]),
         (inputnode, ds_t1w_fsparc, [('t1w_fs_aparc', 'in_file'),
                                     ('source_files', 'source_file')]),
+        (inputnode, ds_anat_ribbon, [('anat_ribbon', 'in_file'),
+                                     ('source_files', 'source_file')]),
+
     ])
     # fmt:on
+
+    if cifti_output:
+        ds_cifti_morph = pe.MapNode(
+            DerivativesDataSink(
+                base_directory=output_dir,
+                suffix=['curv', 'sulc', 'thickness'],
+                compress=False,
+                space='fsLR',
+            ),
+            name='ds_cifti_morph',
+            run_without_submitting=True,
+            iterfield=["in_file", "meta_dict", "suffix"],
+        )
+        # fmt:off
+        workflow.connect([
+            (inputnode, ds_cifti_morph, [('cifti_morph', 'in_file'),
+                                         ('source_files', 'source_file'),
+                                         ('cifti_density', 'density'),
+                                         (('cifti_metadata', _read_jsons), 'meta_dict')])
+        ])
+        # fmt:on
     return workflow
 
 
@@ -752,3 +829,10 @@ def _combine_cohort(in_template):
             return template
         return f"{template}+{in_template.split('cohort-')[-1].split(':')[0]}"
     return [_combine_cohort(v) for v in in_template]
+
+
+def _read_jsons(in_file):
+    from json import loads
+    from pathlib import Path
+
+    return [loads(Path(f).read_text()) for f in in_file]
