@@ -27,25 +27,28 @@ Surface preprocessing workflows.
 structural images.
 
 """
+import typing as ty
 from nipype.pipeline import engine as pe
 from nipype.interfaces.base import Undefined
 from nipype.interfaces import (
+    fsl,
     io as nio,
     utility as niu,
     freesurfer as fs,
+    workbench as wb,
 )
 
-from ..interfaces.freesurfer import ReconAll
+from ..interfaces.freesurfer import ReconAll, MakeMidthickness
 
 from niworkflows.engine.workflows import LiterateWorkflow as Workflow
 from niworkflows.interfaces.freesurfer import (
     FSDetectInputs,
     FSInjectBrainExtracted,
-    MakeMidthickness,
     PatchedLTAConvert as LTAConvert,
     PatchedRobustRegister as RobustRegister,
     RefineBrainMask,
 )
+from ..interfaces.workbench import CreateSignedDistanceVolume
 
 
 def init_surface_recon_wf(*, omp_nthreads, hires, name="surface_recon_wf"):
@@ -377,6 +380,16 @@ def init_autorecon_resume_wf(*, omp_nthreads, name="autorecon_resume_wf"):
         niu.IdentityInterface(fields=["subjects_dir", "subject_id"]), name="outputnode"
     )
 
+    # FreeSurfer 7.3 removed gcareg from autorecon2-volonly
+    # Adding it directly in would force it to run every time
+    gcareg = pe.Node(
+        ReconAll(directive=Undefined, steps=["gcareg"], openmp=omp_nthreads),
+        n_procs=omp_nthreads,
+        mem_gb=5,
+        name="gcareg",
+    )
+    gcareg.interface._always_run = True
+
     autorecon2_vol = pe.Node(
         ReconAll(directive="autorecon2-volonly", openmp=omp_nthreads),
         n_procs=omp_nthreads,
@@ -453,8 +466,10 @@ def init_autorecon_resume_wf(*, omp_nthreads, name="autorecon_resume_wf"):
     workflow.connect([
         (inputnode, cortribbon, [('use_T2', 'use_T2'),
                                  ('use_FLAIR', 'use_FLAIR')]),
-        (inputnode, autorecon2_vol, [('subjects_dir', 'subjects_dir'),
-                                     ('subject_id', 'subject_id')]),
+        (inputnode, gcareg, [('subjects_dir', 'subjects_dir'),
+                             ('subject_id', 'subject_id')]),
+        (gcareg, autorecon2_vol, [('subjects_dir', 'subjects_dir'),
+                                  ('subject_id', 'subject_id')]),
         (autorecon2_vol, autorecon_surfs, [('subjects_dir', 'subjects_dir'),
                                            ('subject_id', 'subject_id')]),
         (autorecon_surfs, cortribbon, [(('subjects_dir', _dedup), 'subjects_dir'),
@@ -663,6 +678,351 @@ def init_segs_to_native_wf(*, name="segs_to_native", segmentation="aseg"):
     return workflow
 
 
+def init_anat_ribbon_wf(name="anat_ribbon_wf"):
+    DEFAULT_MEMORY_MIN_GB = 0.01
+    workflow = pe.Workflow(name=name)
+
+    inputnode = pe.Node(
+        niu.IdentityInterface(
+            fields=[
+                "surfaces",
+                "t1w_mask",
+            ]
+        ),
+        name="inputnode",
+    )
+    outputnode = pe.Node(
+        niu.IdentityInterface(
+            fields=[
+                "anat_ribbon",
+            ]
+        ),
+        name="outputnode",
+    )
+
+    # 0, 1 = wm; 2, 3 = pial; 6, 7 = mid
+    # note that order of lh / rh within each surf type is not guaranteed due to use
+    # of unsorted glob by FreeSurferSource prior, but we can do a sort
+    # to ensure consistent ordering
+    select_wm = pe.Node(
+        niu.Select(index=[0, 1]),
+        name="select_wm",
+        mem_gb=DEFAULT_MEMORY_MIN_GB,
+        run_without_submitting=True,
+    )
+
+    select_pial = pe.Node(
+        niu.Select(index=[2, 3]),
+        name="select_pial",
+        mem_gb=DEFAULT_MEMORY_MIN_GB,
+        run_without_submitting=True,
+    )
+
+    select_midthick = pe.Node(
+        niu.Select(index=[6, 7]),
+        name="select_midthick",
+        mem_gb=DEFAULT_MEMORY_MIN_GB,
+        run_without_submitting=True,
+    )
+
+    create_wm_distvol = pe.MapNode(
+        CreateSignedDistanceVolume(),
+        iterfield=["surf_file"],
+        name="create_wm_distvol",
+    )
+
+    create_pial_distvol = pe.MapNode(
+        CreateSignedDistanceVolume(),
+        iterfield=["surf_file"],
+        name="create_pial_distvol",
+    )
+
+    thresh_wm_distvol = pe.MapNode(
+        fsl.maths.MathsCommand(args="-thr 0 -bin -mul 255"),
+        iterfield=["in_file"],
+        name="thresh_wm_distvol",
+        mem_gb=DEFAULT_MEMORY_MIN_GB,
+    )
+
+    uthresh_pial_distvol = pe.MapNode(
+        fsl.maths.MathsCommand(args="-uthr 0 -abs -bin -mul 255"),
+        iterfield=["in_file"],
+        name="uthresh_pial_distvol",
+        mem_gb=DEFAULT_MEMORY_MIN_GB,
+    )
+
+    bin_wm_distvol = pe.MapNode(
+        fsl.maths.UnaryMaths(operation="bin"),
+        iterfield=["in_file"],
+        name="bin_wm_distvol",
+        mem_gb=DEFAULT_MEMORY_MIN_GB,
+    )
+
+    bin_pial_distvol = pe.MapNode(
+        fsl.maths.UnaryMaths(operation="bin"),
+        iterfield=["in_file"],
+        name="bin_pial_distvol",
+        mem_gb=DEFAULT_MEMORY_MIN_GB,
+    )
+
+    split_wm_distvol = pe.Node(
+        niu.Split(splits=[1, 1]),
+        name="split_wm_distvol",
+        mem_gb=DEFAULT_MEMORY_MIN_GB,
+        run_without_submitting=True,
+    )
+
+    merge_wm_distvol_no_flatten = pe.Node(
+        niu.Merge(2),
+        no_flatten=True,
+        name="merge_wm_distvol_no_flatten",
+        mem_gb=DEFAULT_MEMORY_MIN_GB,
+        run_without_submitting=True,
+    )
+
+    make_ribbon_vol = pe.MapNode(
+        fsl.maths.MultiImageMaths(op_string="-mas %s -mul 255"),
+        iterfield=["in_file", "operand_files"],
+        name="make_ribbon_vol",
+        mem_gb=DEFAULT_MEMORY_MIN_GB,
+    )
+
+    bin_ribbon_vol = pe.MapNode(
+        fsl.maths.UnaryMaths(operation="bin"),
+        iterfield=["in_file"],
+        name="bin_ribbon_vol",
+        mem_gb=DEFAULT_MEMORY_MIN_GB,
+    )
+
+    split_squeeze_ribbon_vol = pe.Node(
+        niu.Split(splits=[1, 1], squeeze=True),
+        name="split_squeeze_ribbon",
+        mem_gb=DEFAULT_MEMORY_MIN_GB,
+        run_without_submitting=True,
+    )
+
+    combine_ribbon_vol_hemis = pe.Node(
+        fsl.maths.BinaryMaths(operation="add"),
+        name="combine_ribbon_vol_hemis",
+        mem_gb=DEFAULT_MEMORY_MIN_GB,
+    )
+
+    # make HCP-style ribbon volume in T1w space
+    workflow.connect(
+        [
+            (inputnode, select_wm, [("surfaces", "inlist")]),
+            (inputnode, select_pial, [("surfaces", "inlist")]),
+            (inputnode, select_midthick, [("surfaces", "inlist")]),
+            (select_wm, create_wm_distvol, [(("out", _sorted_by_basename), "surf_file")]),
+            (inputnode, create_wm_distvol, [("t1w_mask", "ref_file")]),
+            (select_pial, create_pial_distvol, [(("out", _sorted_by_basename), "surf_file")]),
+            (inputnode, create_pial_distvol, [("t1w_mask", "ref_file")]),
+            (create_wm_distvol, thresh_wm_distvol, [("out_file", "in_file")]),
+            (create_pial_distvol, uthresh_pial_distvol, [("out_file", "in_file")]),
+            (thresh_wm_distvol, bin_wm_distvol, [("out_file", "in_file")]),
+            (uthresh_pial_distvol, bin_pial_distvol, [("out_file", "in_file")]),
+            (bin_wm_distvol, split_wm_distvol, [("out_file", "inlist")]),
+            (split_wm_distvol, merge_wm_distvol_no_flatten, [("out1", "in1")]),
+            (split_wm_distvol, merge_wm_distvol_no_flatten, [("out2", "in2")]),
+            (bin_pial_distvol, make_ribbon_vol, [("out_file", "in_file")]),
+            (merge_wm_distvol_no_flatten, make_ribbon_vol, [("out", "operand_files")]),
+            (make_ribbon_vol, bin_ribbon_vol, [("out_file", "in_file")]),
+            (bin_ribbon_vol, split_squeeze_ribbon_vol, [("out_file", "inlist")]),
+            (split_squeeze_ribbon_vol, combine_ribbon_vol_hemis, [("out1", "in_file")]),
+            (split_squeeze_ribbon_vol, combine_ribbon_vol_hemis, [("out2", "operand_file")]),
+            (combine_ribbon_vol_hemis, outputnode, [("out_file", "anat_ribbon")]),
+        ]
+    )
+    return workflow
+
+
+def init_morph_grayords_wf(
+    grayord_density: ty.Literal['91k', '170k'],
+    name: str = "bold_grayords_wf",
+):
+    """
+    Sample Grayordinates files onto the fsLR atlas.
+
+    Outputs are in CIFTI2 format.
+
+    Workflow Graph
+        .. workflow::
+            :graph2use: colored
+            :simple_form: yes
+
+            from smriprep.workflows.surfaces import init_morph_grayords_wf
+            wf = init_morph_grayords_wf(grayord_density="91k")
+
+    Parameters
+    ----------
+    grayord_density : :obj:`str`
+        Either `91k` or `170k`, representing the total of vertices or *grayordinates*.
+    name : :obj:`str`
+        Unique name for the subworkflow (default: ``"bold_grayords_wf"``)
+
+    Inputs
+    ------
+    subject_id : :obj:`str`
+        FreeSurfer subject ID
+    subjects_dir : :obj:`str`
+        FreeSurfer SUBJECTS_DIR
+
+    Outputs
+    -------
+    cifti_morph : :obj:`list` of :obj:`str`
+        Paths of CIFTI dscalar files
+    cifti_metadata : :obj:`list` of :obj:`str`
+        Paths to JSON files containing metadata corresponding to ``cifti_morph``
+
+    """
+    import templateflow.api as tf
+    from niworkflows.engine.workflows import LiterateWorkflow as Workflow
+    from smriprep.interfaces.cifti import GenerateDScalar
+
+    workflow = Workflow(name=name)
+    workflow.__desc__ = f"""\
+*Grayordinate* "dscalar" files [@hcppipelines] containing {grayord_density} samples were
+also generated using the highest-resolution ``fsaverage`` as an intermediate standardized
+surface space.
+"""
+
+    fslr_density = "32k" if grayord_density == "91k" else "59k"
+
+    inputnode = pe.Node(
+        niu.IdentityInterface(fields=["subject_id", "subjects_dir"]),
+        name="inputnode",
+    )
+
+    outputnode = pe.Node(
+        niu.IdentityInterface(fields=["cifti_morph", "cifti_metadata"]),
+        name="outputnode",
+    )
+
+    get_surfaces = pe.Node(nio.FreeSurferSource(), name="get_surfaces")
+
+    surfmorph_list = pe.Node(
+        niu.Merge(3, ravel_inputs=True),
+        name="surfmorph_list",
+        run_without_submitting=True,
+    )
+
+    surf2surf = pe.MapNode(
+        fs.SurfaceTransform(target_subject="fsaverage", target_type="gii"),
+        iterfield=["source_file", "hemi"],
+        name="surf2surf",
+        mem_gb=0.01,
+    )
+    surf2surf.inputs.hemi = ["lh", "rh"] * 3
+
+    # Setup Workbench command. LR ordering for hemi can be assumed, as it is imposed
+    # by the iterfield of the MapNode in the surface sampling workflow above.
+    resample = pe.MapNode(
+        wb.MetricResample(method="ADAP_BARY_AREA", area_metrics=True),
+        name="resample",
+        iterfield=[
+            "in_file",
+            "out_file",
+            "new_sphere",
+            "new_area",
+            "current_sphere",
+            "current_area",
+        ],
+    )
+    resample.inputs.current_sphere = [
+        str(
+            tf.get(
+                "fsaverage",
+                hemi=hemi,
+                density="164k",
+                desc="std",
+                suffix="sphere",
+                extension=".surf.gii",
+            )
+        )
+        for hemi in "LR"
+    ] * 3
+    resample.inputs.current_area = [
+        str(
+            tf.get(
+                "fsaverage",
+                hemi=hemi,
+                density="164k",
+                desc="vaavg",
+                suffix="midthickness",
+                extension=".shape.gii",
+            )
+        )
+        for hemi in "LR"
+    ] * 3
+    resample.inputs.new_sphere = [
+        str(
+            tf.get(
+                "fsLR",
+                space="fsaverage",
+                hemi=hemi,
+                density=fslr_density,
+                suffix="sphere",
+                extension=".surf.gii",
+            )
+        )
+        for hemi in "LR"
+    ] * 3
+    resample.inputs.new_area = [
+        str(
+            tf.get(
+                "fsLR",
+                hemi=hemi,
+                density=fslr_density,
+                desc="vaavg",
+                suffix="midthickness",
+                extension=".shape.gii",
+            )
+        )
+        for hemi in "LR"
+    ] * 3
+    resample.inputs.out_file = [
+        f"space-fsLR_hemi-{h}_den-{grayord_density}_{morph}.shape.gii"
+        # Order: curv-L, curv-R, sulc-L, sulc-R, thickness-L, thickness-R
+        for morph in ('curv', 'sulc', 'thickness')
+        for h in "LR"
+    ]
+
+    gen_cifti = pe.MapNode(
+        GenerateDScalar(
+            grayordinates=grayord_density,
+        ),
+        iterfield=['scalar_name', 'scalar_surfs'],
+        name="gen_cifti",
+    )
+    gen_cifti.inputs.scalar_name = ['curv', 'sulc', 'thickness']
+
+    # fmt: off
+    workflow.connect([
+        (inputnode, get_surfaces, [
+            ('subject_id', 'subject_id'),
+            ('subjects_dir', 'subjects_dir'),
+        ]),
+        (inputnode, surf2surf, [
+            ('subject_id', 'source_subject'),
+            ('subjects_dir', 'subjects_dir'),
+        ]),
+        (get_surfaces, surfmorph_list, [
+            (('curv', _sorted_by_basename), 'in1'),
+            (('sulc', _sorted_by_basename), 'in2'),
+            (('thickness', _sorted_by_basename), 'in3'),
+        ]),
+        (surfmorph_list, surf2surf, [('out', 'source_file')]),
+        (surf2surf, resample, [('out_file', 'in_file')]),
+        (resample, gen_cifti, [
+            (("out_file", _collate), "scalar_surfs")]),
+        (gen_cifti, outputnode, [("out_file", "cifti_morph"),
+                                 ("out_metadata", "cifti_metadata")]),
+    ])
+    # fmt: on
+
+    return workflow
+
+
 def _check_cw256(in_files, default_flags):
     import numpy as np
     from nibabel.funcs import concat_images
@@ -675,3 +1035,13 @@ def _check_cw256(in_files, default_flags):
     if np.any(fov > 256):
         flags.append("-cw256")
     return flags
+
+
+def _sorted_by_basename(inlist):
+    from os.path import basename
+
+    return sorted(inlist, key=lambda x: str(basename(x)))
+
+
+def _collate(files):
+    return [files[i:i + 2] for i in range(0, len(files), 2)]
